@@ -500,53 +500,6 @@ static void helpOption(char *arg)
 	exit(0);
 }
 
-/*
- * Length-delimited string comparison with ignore case.
- */
-static int StrNICmp(const char *str1, const char *str2, int len)
-{
-	char ch1;
-	char ch2;
-	while(len > 0 && *str1 != '\0' && *str2 != '\0')
-	{
-		ch1 = *str1++;
-		if(ch1 >= 'A' && ch1 <= 'Z')
-		{
-			ch1 = (ch1 - 'A' + 'a');
-		}
-		ch2 = *str2++;
-		if(ch2 >= 'A' && ch2 <= 'Z')
-		{
-			ch2 = (ch2 - 'A' + 'a');
-		}
-		if(ch1 < ch2)
-		{
-			return -1;
-		}
-		else if(ch1 > ch2)
-		{
-			return 1;
-		}
-		--len;
-	}
-	if(!len)
-	{
-		return 0;
-	}
-	if(*str1 != '\0')
-	{
-		return 1;
-	}
-	else if(*str2 != '\0')
-	{
-		return -1;
-	}
-	else
-	{
-		return 0;
-	}
-}
-
 /*
  * Determine if a command-line looks like something that should
  * be processed using the compatibility parser.
@@ -592,7 +545,7 @@ int CCNeedsCompatParser(int argc, char *argv[])
 			}
 			if(opt->nameSize)
 			{
-				if(!StrNICmp(argv[posn], opt->name, opt->nameSize) &&
+				if(!ILStrNICmp(argv[posn], opt->name, opt->nameSize) &&
 				   (argv[posn][opt->nameSize] == ':' ||
 				    argv[posn][opt->nameSize] == '=' ||
 				    argv[posn][opt->nameSize] == '\0'))
@@ -627,6 +580,28 @@ static void InvokeOption(const CmdLineOpt *opt, char *arg)
 	}
 }
 
+/*
+ * Determine if a string starting with '/' looks like an absolute
+ * Unix-style pathname.
+ */
+static int LooksLikeAbsolutePath(const char *str)
+{
+	++str;
+	while(*str != '\0')
+	{
+		if(*str == '/')
+		{
+			return 1;
+		}
+		else if(*str == ':' || *str == '=')
+		{
+			break;
+		}
+		++str;
+	}
+	return 0;
+}
+
 /*
  * Parse the command-line options with the compatibility parser.
  */
@@ -645,6 +620,11 @@ void CCParseWithCompatParser(int argc, char *argv[])
 			CCStringListAdd(&input_files, &num_input_files, argv[posn]);
 			continue;
 		}
+		if(argv[posn][0] == '/' && LooksLikeAbsolutePath(argv[posn]))
+		{
+			CCStringListAdd(&input_files, &num_input_files, argv[posn]);
+			continue;
+		}
 
 		/* Search for the option in "compatOptions" */
 		opt = compatOptions;
@@ -657,14 +637,14 @@ void CCParseWithCompatParser(int argc, char *argv[])
 			}
 			if(size)
 			{
-				if(!StrNICmp(argv[posn], opt->name, size) &&
+				if(!ILStrNICmp(argv[posn], opt->name, size) &&
 				   (argv[posn][size] == ':' || argv[posn][size] == '='))
 				{
 					/* This is an option with arguments */
 					InvokeOption(opt, argv[posn] + size + 1);
 					break;
 				}
-				else if(!StrNICmp(argv[posn], opt->name, size) &&
+				else if(!ILStrNICmp(argv[posn], opt->name, size) &&
 				        argv[posn][size] != '\0' && opt->nameSize < 0)
 				{
 					/* Option with arguments, cscc-style */
