@@ -48,16 +48,16 @@ extern	"C" {
  */
 #define	INCLUDE_PATH	\
 			"/usr/local/lib/cscc/lib:" \
-			"/usr/local/lib/cscc/include:" \
+			"/usr/local/share/cscc/include:" \
 			"/usr/lib/cscc/lib:" \
-			"/usr/lib/cscc/include"
+			"/usr/share/cscc/include"
 
 /*
  * The default system C++ include path.
  */
 #define	INCLUDE_CPP_PATH	\
-			"/usr/local/lib/cscc/include/c++:" \
-			"/usr/lib/cscc/include/c++"
+			"/usr/local/share/cscc/include/c++:" \
+			"/usr/share/cscc/include/c++"
 
 /*
  * The default system library link path.
@@ -195,6 +195,25 @@ int main(int argc, char *argv[])
 					language = "cs";
 					extension = "cs";
 				}
+				else if(CompareExtensions(filename + len, "vb"))
+				{
+					/* This is a VB source file */
+					language = "vb";
+					extension = "vb";
+				}
+				else if(CompareExtensions(filename + len, "java"))
+				{
+					/* This is a Java source file */
+					language = "java";
+					extension = "java";
+				}
+				else if(CompareExtensions(filename + len, "bf") ||
+						CompareExtensions(filename + len, "b"))
+				{
+					/* This is a bf source file */
+					language = "bf";
+					extension = filename + len;
+				}
 				else
 				{
 					/* Use the extension as the language name */
@@ -402,7 +421,7 @@ static void ParseCommandLine(int argc, char *argv[])
 			CCAddPathStrings(&sys_include_dirs, &num_sys_include_dirs,
 							 INCLUDE_PATH,
 							 ILGetStandardLibraryPath("cscc/lib"),
-							 ILGetStandardLibraryPath("cscc/include"));
+							 ILGetStandardDataPath("cscc/include"));
 		}
 	}
 	if(!nostdinc_cpp_flag)
@@ -417,7 +436,7 @@ static void ParseCommandLine(int argc, char *argv[])
 		{
 			CCAddPathStrings(&sys_cpp_include_dirs, &num_sys_cpp_include_dirs,
 							 INCLUDE_CPP_PATH,
-							 ILGetStandardLibraryPath("cscc/include/c++"), 0);
+							 ILGetStandardDataPath("cscc/include/c++"), 0);
 		}
 	}
 
@@ -450,14 +469,18 @@ static void ParseCommandLine(int argc, char *argv[])
 	{
 		if(executable_flag)
 		{
-			/* The default output filename is "a.out.dll" or "a.out.exe" */
+			/* The default output filename is "a.dll" or "a.exe/a.out" */
 			if(shared_flag)
 			{
-				output_filename = "a.out.dll";
+				output_filename = "a.dll";
 			}
 			else
 			{
-				output_filename = "a.out.exe";
+			#ifdef IL_WIN32_PLATFORM
+				output_filename = "a.exe";
+			#else
+				output_filename = "a.out";
+			#endif
 			}
 		}
 		else if(compile_flag)
@@ -517,6 +540,18 @@ static void ParseCommandLine(int argc, char *argv[])
 		{
 			prog_language_name = "c";
 		}
+		else if(prog_language == PROG_LANG_VB)
+		{
+			prog_language_name = "vb";
+		}
+		else if(prog_language == PROG_LANG_JAVA)
+		{
+			prog_language_name = "java";
+		}
+		else if(prog_language == PROG_LANG_BF)
+		{
+			prog_language_name = "bf";
+		}
 		else
 		{
 			prog_language_name = "cs";
@@ -873,7 +908,7 @@ static int IsSinglePlugin(const char *filename)
 	/* Determine if the executable name ends in "-s" */
 	if(len > 2 &&
 	   (filename[len - 1] == 's' || filename[len - 1] == 'S') &&
-	   filename[len - 2] == '.')
+	   filename[len - 2] == '-')
 	{
 		return 1;
 	}
@@ -922,33 +957,10 @@ static void DumpCmdLine(char **argv)
 	}
 }
 
-/*
- * Execute a child process and wait for it to exit.
- * Returns the status code.
- */
-static int ExecChild(char **argv, const char *filename)
-{
-	int status;
-
-	/* Dump the command-line if we are in verbose mode */
-	DumpCmdLine(argv);
-
-	/* Use the system-specifc process spawn routine */
-	status = ILSpawnProcess(argv);
-	if(status < 0)
-	{
-		return 1;
-	}
-	else
-	{
-		return status;
-	}
-}
-
 /*
  * Import the assembler code from "libILAsm".
  */
-int ILAsmMain(int argc, char *argv[]);
+int ILAsmMain(int argc, char *argv[], FILE *newStdin);
 
 /*
  * Process an input file using the assembler.
@@ -1003,8 +1015,9 @@ static int ProcessWithAssembler(const char *filename, int jvmMode)
 	AddArgument(&cmdline, &cmdline_size, 0);
 
 	/* Execute the assembler */
+	DumpCmdLine(cmdline);
 	ILCmdLineSuppressSlash();
-	status = ILAsmMain(cmdline_size - 1, cmdline);
+	status = ILAsmMain(cmdline_size - 1, cmdline, 0);
 	ILFree(cmdline);
 	if(status != 0)
 	{
@@ -1026,6 +1039,11 @@ static int ProcessWithPlugin(const char *filename, char *plugin,
 	char *asm_output;
 	char *obj_output;
 	int saveAsm;
+	int outputIndex = -1;
+	FILE *newStdin = 0;
+	int pipePid = 0;
+	int canPipe;
+	char **pluginCmdline = 0;
 
 	/* Build the command-line for the plug-in */
 	cmdline = 0;
@@ -1186,6 +1204,7 @@ static int ProcessWithPlugin(const char *filename, char *plugin,
 	else
 	{
 		AddArgument(&cmdline, &cmdline_size, "-o");
+		outputIndex = cmdline_size;
 		if(assemble_flag)
 		{
 			if(output_filename)
@@ -1247,9 +1266,64 @@ static int ProcessWithPlugin(const char *filename, char *plugin,
 	/* Terminate the command-line */
 	AddArgument(&cmdline, &cmdline_size, 0);
 
-	/* Execute the plugin */
-	status = ExecChild(cmdline, filename);
-	ILFree(cmdline);
+	/* Determine if we need to save the assembly stream */
+	saveAsm = CCStringListContains(extension_flags, num_extension_flags,
+							       "save-asm");
+
+	/* Determine if we might be able to pipe the output into the assembler */
+	canPipe = 1;
+	if(assemble_flag || preprocess_flag || saveAsm)
+	{
+		canPipe = 0;
+	}
+	if(CCStringListContains(extension_flags, num_extension_flags,
+							"syntax-check"))
+	{
+		canPipe = 0;
+	}
+
+	/* Execute the plugin, using a pipe if possible */
+	if(canPipe)
+	{
+		cmdline[outputIndex] = "-";
+		DumpCmdLine(cmdline);
+		status = ILSpawnProcessWithPipe(cmdline, (void **)&newStdin);
+		cmdline[outputIndex] = asm_output;
+		if(status < 0)
+		{
+			/* Failed to spawn the process */
+			status = 1;
+			ILFree(cmdline);
+		}
+		else if(!status)
+		{
+			/* This platform does not support pipes: fall back */
+			status = ILSpawnProcess(cmdline);
+			if(status < 0)
+			{
+				status = 1;
+			}
+			ILFree(cmdline);
+		}
+		else
+		{
+			/* We won't have an on-disk assembly output file */
+			asm_output = 0;
+			pipePid = status;
+			status = 0;
+			pluginCmdline = cmdline;
+		}
+	}
+	else
+	{
+		DumpCmdLine(cmdline);
+		status = ILSpawnProcess(cmdline);
+		if(status < 0)
+		{
+			status = 1;
+		}
+		ILFree(cmdline);
+	}
 	if(status != 0)
 	{
 		if(asm_output != 0)
@@ -1268,7 +1342,10 @@ static int ProcessWithPlugin(const char *filename, char *plugin,
 	if(CCStringListContains(extension_flags, num_extension_flags,
 							"syntax-check"))
 	{
-		ILDeleteFile(asm_output);
+		if(asm_output != 0)
+		{
+			ILDeleteFile(asm_output);
+		}
 		return 0;
 	}
 
@@ -1306,25 +1383,44 @@ static int ProcessWithPlugin(const char *filename, char *plugin,
 		AddArgument(&cmdline, &cmdline_size, machine_flags[posn]);
 	}
 	AddArgument(&cmdline, &cmdline_size, "--");
-	AddArgument(&cmdline, &cmdline_size, asm_output);
+	if(newStdin)
+	{
+		/* Use the pipe output of the plugin as the assembler's input */
+		AddArgument(&cmdline, &cmdline_size, "-");
+	}
+	else
+	{
+		AddArgument(&cmdline, &cmdline_size, asm_output);
+	}
 	AddArgument(&cmdline, &cmdline_size, 0);
 
 	/* Execute the assembler */
-	saveAsm = CCStringListContains(extension_flags, num_extension_flags,
-							       "save-asm");
+	DumpCmdLine(cmdline);
 	ILCmdLineSuppressSlash();
-	status = ILAsmMain(cmdline_size - 1, cmdline);
+	status = ILAsmMain(cmdline_size - 1, cmdline, newStdin);
 	ILFree(cmdline);
+	if(newStdin)
+	{
+		/* Close the pipe to the language plugin */
+		int newStatus;
+		fclose(newStdin);
+		newStatus = ILSpawnProcessWaitForExit(pipePid, cmdline);
+		ILFree(pluginCmdline);
+		if(status == 0)
+		{
+			status = newStatus;
+		}
+	}
 	if(status != 0)
 	{
-		if(!saveAsm)
+		if(!saveAsm && asm_output)
 		{
 			ILDeleteFile(asm_output);
 		}
 		ILDeleteFile(obj_output);
 		return status;
 	}
-	if(!saveAsm)
+	if(!saveAsm && asm_output)
 	{
 		ILDeleteFile(asm_output);
 	}
