@@ -79,6 +79,45 @@ static ILClass *NodeToClass(ILNode *node)
 	return 0;
 }
 
+/*
+ * Get the full and basic names from a method/property/event name.
+ */
+static char *GetFullAndBasicNames(ILNode *name, char **basicName)
+{
+	char *result;
+	char *basic;
+	char *left;
+	if(yyisa(name, ILNode_Identifier))
+	{
+		result = ILQualIdentName(name, 0);
+		basic = result;
+	}
+	else if(yyisa(name, ILNode_GenericReference))
+	{
+		result = ILQualIdentName(((ILNode_GenericReference *)name)->type, 0);
+		basic = result;
+	}
+	else if(yyisa(name, ILNode_QualIdent))
+	{
+		left = GetFullAndBasicNames(((ILNode_QualIdent *)name)->left, 0);
+		result = GetFullAndBasicNames
+			(((ILNode_QualIdent *)name)->right, &basic);
+		result = ILQualIdentAppend(left, result);
+	}
+	else
+	{
+		/* Shouldn't happen, but do something safe */
+		CCErrorOnLine(yygetfilename(name), yygetlinenum(name),
+					  _("invalid qualified identifier"));
+		result = basic = "x";
+	}
+	if(basicName)
+	{
+		*basicName = basic;
+	}
+	return result;
+}
+
 /*
  * Create the program structure for a type and all of its base types.
  * Returns the new end of the top-level type list.
@@ -89,8 +128,6 @@ static void CreateType(ILGenInfo *info, ILScope *globalScope,
 {
 	const char *name;
 	const char *namespace;
-	const char *baseName;
-	int namelen;
 	int numBases;
 	ILClass **baseList;
 	int base;
@@ -103,9 +140,13 @@ static void CreateType(ILGenInfo *info, ILScope *globalScope,
 	ILNode_ClassDefn *defn;
 	ILNode *savedNamespace;
 	ILNode *savedClass;
+	ILNode *savedTypeFormals;
 	ILProgramItem *nestedScope;
 	ILNode *node;
 	ILNode_ListIter iter;
+	ILUInt32 formalNum;
+	ILGenericPar *genPar;
+	ILClass *underlying;
 
 	/* Get the name and namespace for the type, for error reporting */
 	defn = (ILNode_ClassDefn *)type;
@@ -163,6 +204,8 @@ static void CreateType(ILGenInfo *info, ILScope *globalScope,
 	info->currentNamespace = defn->namespaceNode;
 	savedClass = info->currentClass;
 	info->currentClass = (ILNode *)(defn->nestedParent);
+	savedTypeFormals = info->currentTypeFormals;
+	info->currentTypeFormals = defn->typeFormals;
 
 	/* Create all of the base classes */
 	numBases = CountBaseClasses(defn->baseClass);
@@ -192,35 +235,6 @@ static void CreateType(ILGenInfo *info, ILScope *globalScope,
 			baseNode = baseNodeList;
 		}
 
-		if(yyisa(baseNode,ILNode_Identifier))
-		{
-			baseName=ILQualIdentName(baseNode,0);
-		}
-		else if(yyisa(baseNode,ILNode_QualIdent))
-		{
-			baseName=ILQualIdentName(((ILNode_QualIdent*)baseNode)->right,0);
-		}
-		else
-		{
-			baseName=0;
-		}
-	
-		if(baseName)
-		{
-			namelen=strlen(baseName);
-			if(namelen >= 9 && strcmp(baseName + namelen - 9, "Attribute") == 0)
-			{
-				namelen=strlen(name);
-				if(namelen < 9 || strcmp(name + namelen - 9, "Attribute") != 0)
-				{
-					defn->name = ILInternAppendedString
-									(ILInternString(defn->name, namelen),
-									 ILInternString("Attribute", 9)).string;
-					name=defn->name;
-				}
-			}
-		}
-
 		/* Look in the scope for the base class */
 		if(CSSemBaseType(baseNode, info, &baseNode,
 						 &baseTypeNode, &(baseList[base])))
@@ -318,12 +332,25 @@ static void CreateType(ILGenInfo *info, ILScope *globalScope,
 		}
 	}
 
-	/* Restore the namespace and class */
+	/* Restore the namespace, class, and type formals */
 	info->currentNamespace = savedNamespace;
 	info->currentClass = savedClass;
+	info->currentTypeFormals = savedTypeFormals;
 
 	/* Output an error if attempting to inherit from a sealed class */
-	if(parent && ILClass_IsSealed(parent))
+	if(parent)
+	{
+		underlying = ILClassGetUnderlying(parent);
+		if(!underlying)
+		{
+			CCOutOfMemory();
+		}
+	}
+	else
+	{
+		underlying = 0;
+	}
+	if(underlying && ILClass_IsSealed(underlying))
 	{
 		CCErrorOnLine(yygetfilename(type), yygetlinenum(type),
 					  "inheriting from a sealed parent class");
@@ -358,6 +385,24 @@ static void CreateType(ILGenInfo *info, ILScope *globalScope,
 		}
 	}
 
+	/* Add the type formals to the class definition */
+	ILNode_ListIter_Init(&iter, defn->typeFormals);
+	formalNum = 0;
+	while((node = ILNode_ListIter_Next(&iter)) != 0)
+	{
+		genPar = ILGenericParCreate
+			(info->image, 0, ILToProgramItem(classInfo), formalNum);
+		if(!genPar)
+		{
+			CCOutOfMemory();
+		}
+		if(!ILGenericParSetName(genPar, ILQualIdentName(node, 0)))
+		{
+			CCOutOfMemory();
+		}
+		++formalNum;
+	}
+
 	/* Clean up */
 	if(baseList)
 	{
@@ -416,10 +461,10 @@ static ILMember *FindMemberByName(ILClass *classInfo, const char *name,
 	{
 		/* Scan the members of this class */
 		member = 0;
-		while((member = ILClassNextMember(classInfo, member)) != 0)
+		while((member = ILClassNextMemberMatch
+				(classInfo, member, 0, name, 0)) != 0)
 		{
-			if(!strcmp(ILMember_Name(member), name) &&
-			   ILMemberAccessible(member, scope))
+			if(ILMemberAccessible(member, scope))
 			{
 				return member;
 			}
@@ -462,10 +507,10 @@ static ILMember *FindMemberBySignature(ILClass *classInfo, const char *name,
 	{
 		/* Scan the members of this class */
 		member = 0;
-		while((member = ILClassNextMember(classInfo, member)) != 0)
+		while((member = ILClassNextMemberMatch
+				(classInfo, member, 0, name, 0)) != 0)
 		{
 			if(member != notThis &&
-			   !strcmp(ILMember_Name(member), name) &&
 			   ILMemberAccessible(member, scope) &&
 			   (!interfaceOverride || classInfo == scope))
 			{
@@ -640,12 +685,9 @@ static ILMember *FindInterfaceMatch(ILClass *interface,
 									int kind)
 {
 	ILMember *member = 0;
-	while((member = ILClassNextMemberByKind(interface, member, kind)) != 0)
+	while((member = ILClassNextMemberMatch
+			(interface, member, kind, name, 0)) != 0)
 	{
-		if(strcmp(ILMember_Name(member), name) != 0)
-		{
-			continue;
-		}
 		if(kind == IL_META_MEMBERKIND_METHOD ||
 		   kind == IL_META_MEMBERKIND_PROPERTY)
 		{
@@ -845,6 +887,46 @@ static char *GetFullExplicitName(ILClass *interface, char *memberName)
 	return name;
 }
 
+/*
+ * Determine if a declaration of "Finalize" will override
+ * the one in "Object" or if it is on a separate "new" slot.
+ */
+static int IsRealFinalizer(ILClass *classInfo)
+{
+	ILClass *parent = ILClass_Parent(classInfo);
+	ILMethod *method;
+	ILType *signature;
+	while(parent != 0)
+	{
+		if(ILTypeIsObjectClass(ILType_FromClass(parent)))
+		{
+			/* We've found the declaration in "System.Object" */
+			return 1;
+		}
+		method = 0;
+		while((method = (ILMethod *)ILClassNextMemberMatch
+				(parent, (ILMember *)method,
+				 IL_META_MEMBERKIND_METHOD, "Finalize", 0)) != 0)
+		{
+			signature = ILMethod_Signature(method);
+			if(ILTypeGetReturn(signature) == ILType_Void &&
+			   ILTypeNumParams(signature) == 0)
+			{
+				if((ILMethod_Attrs(method) &
+						(IL_META_METHODDEF_NEW_SLOT |
+						 IL_META_METHODDEF_VIRTUAL)) !=
+					 IL_META_METHODDEF_VIRTUAL)
+				{
+					/* We've found something other than the real "Finalize" */
+					return 0;
+				}
+			}
+		}
+		parent = ILClass_Parent(parent);
+	}
+	return 0;
+}
+
 /*
  * Create a method definition.
  */
@@ -870,18 +952,16 @@ static void CreateMethod(ILGenInfo *info, ILClass *classInfo,
 	/* Get the name of the method, and the interface member (if any) */
 	interface = 0;
 	interfaceMember = 0;
-	if(yykind(method->name) == yykindof(ILNode_Identifier))
+	if(yykind(method->name) == yykindof(ILNode_Identifier) ||
+	   yykind(method->name) == yykindof(ILNode_GenericReference))
 	{
 		/* Simple method name */
-		name = ILQualIdentName(method->name, 0);
-		basicName = name;
+		name = GetFullAndBasicNames(method->name, &basicName);
 	}
 	else
 	{
 		/* Qualified method name that overrides some interface method */
-		name = ILQualIdentName(method->name, 0);
-		basicName = ILQualIdentName
-			(((ILNode_QualIdent *)(method->name))->right, 0);
+		name = GetFullAndBasicNames(method->name, &basicName);
 		signature = CSSemType(((ILNode_QualIdent *)(method->name))->left, info,
 							  &(((ILNode_QualIdent *)(method->name))->left));
 		if(signature)
@@ -919,12 +999,11 @@ static void CreateMethod(ILGenInfo *info, ILClass *classInfo,
 	   !strcmp(ILClass_Namespace(classInfo), "System"))
 	{
 		methodInfo = 0;
-		while((methodInfo = (ILMethod *)ILClassNextMemberByKind
+		while((methodInfo = (ILMethod *)ILClassNextMemberMatch
 					(classInfo, (ILMember *)methodInfo,
-					 IL_META_MEMBERKIND_METHOD)) != 0)
+					 IL_META_MEMBERKIND_METHOD, ".ctor", 0)) != 0)
 		{
-			if(!strcmp(ILMethod_Name(methodInfo), ".ctor") &&
-			   (ILMethod_Token(methodInfo) & IL_META_TOKEN_MASK) ==
+			if((ILMethod_Token(methodInfo) & IL_META_TOKEN_MASK) ==
 			   		IL_META_TOKEN_MEMBER_REF)
 			{
 				if(!ILMethodNewToken(methodInfo))
@@ -956,6 +1035,27 @@ static void CreateMethod(ILGenInfo *info, ILClass *classInfo,
 	/* Get the return type */
 	tempType = CSSemTypeVoid(method->type, info, &(method->type));
 
+	/* Special handling for "Finalize" to be consistent with the ECMA spec */
+	if(!strcmp(name, "Finalize") &&
+	   (method->params == 0 || yyisa(method->params, ILNode_Empty)) &&
+	   tempType == ILType_Void &&
+	   (method->modifiers & CS_SPECIALATTR_DESTRUCTOR) == 0)
+	{
+		if((method->modifiers & CS_SPECIALATTR_OVERRIDE) != 0)
+		{
+			if(IsRealFinalizer(classInfo))
+			{
+				CCErrorOnLine(yygetfilename(method), yygetlinenum(method),
+							  "do not override `Object.Finalize'; provide "
+							  "a destructor instead");
+			}
+		}
+		else if(!ILTypeIsObjectClass(ILType_FromClass(classInfo)))
+		{
+			method->modifiers |= CS_SPECIALATTR_NEW;
+		}
+	}
+
 	/* Create the method signature type */
 	signature = ILTypeCreateMethod(info->context, tempType);
 	if(!signature)
@@ -1093,6 +1193,17 @@ static void CreateMethod(ILGenInfo *info, ILClass *classInfo,
 	   (strncmp(ILMethod_Name(methodInfo), "get_", 4) != 0 &&
 	    strncmp(ILMethod_Name(methodInfo), "set_", 4) != 0))
 	{
+		/* If "override" is supplied, then look for its "virtual" */
+		if((method->modifiers & CS_SPECIALATTR_OVERRIDE) != 0)
+		{
+			if(!ILMemberGetBase((ILMember *)methodInfo))
+			{
+				CCErrorOnLine(yygetfilename(method), yygetlinenum(method),
+							  "`override' used on a method with no "
+							  "corresponding `virtual'");
+			}
+		}
+
 		/* Look for duplicates and report on them */
 		member = FindMemberBySignature(classInfo, name, signature,
 									   (ILMember *)methodInfo, classInfo,
@@ -1104,7 +1215,8 @@ static void CreateMethod(ILGenInfo *info, ILClass *classInfo,
 			   !ILMethod_IsNewSlot(methodInfo))
 			{
 				/* Check for the correct form of virtual method overrides */
-				if((method->modifiers & CS_SPECIALATTR_OVERRIDE) == 0)
+				if((method->modifiers & CS_SPECIALATTR_OVERRIDE) == 0 &&
+				   (method->modifiers & CS_SPECIALATTR_NEW) == 0)
 				{
 					CCWarningOnLine(yygetfilename(method), yygetlinenum(method),
 			  			"declaration of `%s' overrides an inherited member, "
@@ -1131,6 +1243,12 @@ static void CreateMethod(ILGenInfo *info, ILClass *classInfo,
 						ILClass_Name(class2),
 						name);
 				}
+				if(ILMember_Owner(member) == classInfo)
+				{
+					ReportDuplicates(method->name, (ILMember *)methodInfo,
+									 member, classInfo,
+									 method->modifiers, name);
+				}
 			}
 			else if(ILMember_Owner(member) == classInfo ||
 			        (!ILMethodIsConstructor(methodInfo) &&
@@ -1279,19 +1397,17 @@ static void CreateProperty(ILGenInfo *info, ILClass *classInfo,
 	}
 
 	/* Get the name of the property */
-	if(yykind(property->name) == yykindof(ILNode_Identifier))
+	if(yykind(property->name) == yykindof(ILNode_Identifier) ||
+	   yykind(property->name) == yykindof(ILNode_GenericReference))
 	{
 		/* Simple property name */
-		name = ILQualIdentName(property->name, 0);
-		basicName = name;
+		name = GetFullAndBasicNames(property->name, &basicName);
 		interfaceOverride = 0;
 	}
 	else
 	{
 		/* Qualified property name that overrides some interface property */
-		name = ILQualIdentName(property->name, 0);
-		basicName = ILQualIdentName
-			(((ILNode_QualIdent *)(property->name))->right, 0);
+		name = GetFullAndBasicNames(property->name, &basicName);
 		signature = CSSemType
 				(((ILNode_QualIdent *)(property->name))->left, info,
 			     &(((ILNode_QualIdent *)(property->name))->left));
@@ -1328,10 +1444,6 @@ static void CreateProperty(ILGenInfo *info, ILClass *classInfo,
 	{
 		CCOutOfMemory();
 	}
-	if((property->modifiers & IL_META_METHODDEF_STATIC) == 0)
-	{
-		ILTypeSetCallConv(signature, IL_META_CALLCONV_HASTHIS);
-	}
 
 	/* Create the parameters for the property */
 	paramNum = 1;
@@ -1402,6 +1514,17 @@ static void CreateProperty(ILGenInfo *info, ILClass *classInfo,
 	AddMemberToScope(info->currentScope, IL_SCOPE_PROPERTY,
 					 name, (ILMember *)propertyInfo, property->name);
 
+	/* If "override" is supplied, then look for its "virtual" */
+	if((property->modifiers & CS_SPECIALATTR_OVERRIDE) != 0)
+	{
+		if(!ILMemberGetBase((ILMember *)propertyInfo))
+		{
+			CCErrorOnLine(yygetfilename(property), yygetlinenum(property),
+						  "`override' used on a property with no "
+						  "corresponding `virtual'");
+		}
+	}
+
 	/* Look for duplicates and report on them */
 	member = FindMemberBySignature(classInfo, name, signature,
 								   (ILMember *)propertyInfo, classInfo,
@@ -1410,7 +1533,7 @@ static void CreateProperty(ILGenInfo *info, ILClass *classInfo,
 	{
 		if(ILMember_IsProperty(member) &&
 		   PropertyIsVirtual((ILProperty *)member) &&
-		   (property->modifiers & IL_META_METHODDEF_NEW_SLOT) == 0)
+		   (property->modifiers & CS_SPECIALATTR_NEW) == 0)
 		{
 			/* Check for the correct form of virtual method overrides */
 			if((property->modifiers & CS_SPECIALATTR_OVERRIDE) == 0)
@@ -1467,18 +1590,17 @@ static void CreateEventDecl(ILGenInfo *info, ILClass *classInfo,
 
 	/* Get the name of the event */
 	eventName = ((ILNode_FieldDeclarator *)(eventDecl->fieldDeclarator))->name;
-	if(yykind(eventName) == yykindof(ILNode_Identifier))
+	if(yykind(eventName) == yykindof(ILNode_Identifier) ||
+	   yykind(eventName) == yykindof(ILNode_GenericReference))
 	{
 		/* Simple event name */
-		name = ILQualIdentName(eventName, 0);
+		name = GetFullAndBasicNames(eventName, &basicName);
 		interfaceOverride = 0;
 	}
 	else
 	{
 		/* Qualified event name that overrides some interface event */
-		name = ILQualIdentName(eventName, 0);
-		basicName = ILQualIdentName
-			(((ILNode_QualIdent *)eventName)->right, 0);
+		name = GetFullAndBasicNames(eventName, &basicName);
 		signature = CSSemType
 				(((ILNode_QualIdent *)eventName)->left, info,
 			     &(((ILNode_QualIdent *)eventName)->left));
@@ -1554,6 +1676,17 @@ static void CreateEventDecl(ILGenInfo *info, ILClass *classInfo,
 	AddMemberToScope(info->currentScope, IL_SCOPE_EVENT,
 					 name, (ILMember *)eventInfo, eventName);
 
+	/* If "override" is supplied, then look for its "virtual" */
+	if((event->modifiers & CS_SPECIALATTR_OVERRIDE) != 0)
+	{
+		if(!ILMemberGetBase((ILMember *)eventInfo))
+		{
+			CCErrorOnLine(yygetfilename(event), yygetlinenum(event),
+						  "`override' used on an event with no "
+						  "corresponding `virtual'");
+		}
+	}
+
 	/* Report on the duplicates */
 	if(member)
 	{
@@ -1662,6 +1795,45 @@ static void CreateEvent(ILGenInfo *info, ILClass *classInfo,
 	}
 }
 
+/*
+ * Create a formal parameter with a system type
+ */
+static ILNode_FormalParameter* CreateFormalParameter(char *systemtype,
+													 char *name)
+{
+	ILNode *param=ILNode_FormalParameter_create(
+						NULL,ILParamMod_empty, 
+						ILNode_SystemType_create(systemtype),
+		 				ILQualIdentSimple(ILInternString(name, -1).string));
+	return (ILNode_FormalParameter*)param;
+}
+
+/* 
+ * Append parameters in `from' to `to' , the onlyPtrs flag if enabled
+ * appends only out or ref parameters
+ */
+static void AppendParameters(ILNode *from,ILNode_List *to,int onlyPtrs)
+{
+	ILNode_ListIter iter;
+	ILNode_FormalParameter *node;
+	if(!yyisa(from,ILNode_List)) return;
+	ILNode_ListIter_Init(&iter,from);
+	while((node = (ILNode_FormalParameter*)ILNode_ListIter_Next(&iter))!=0)
+	{
+		if((!onlyPtrs) || (node->pmod==ILParamMod_ref || 
+							node->pmod == ILParamMod_out))
+		{
+			ILNode_FormalParameter *param= (ILNode_FormalParameter*)
+											ILNode_FormalParameter_create(NULL,
+													node->pmod,node->type,
+													node->name);
+			yysetfilename((ILNode*)param,yygetfilename(node));
+			yysetlinenum((ILNode*)param, yygetlinenum(node));
+			ILNode_List_Add(to,param);
+		}
+	}
+}
+
 /*
  * Create a delegate member definition.
  */
@@ -1671,6 +1843,7 @@ static void CreateDelegateMember(ILGenInfo *info, ILClass *classInfo,
 	ILMethod *method;
 	ILType *signature;
 	ILNode_MethodDeclaration *decl;
+	ILNode_List *params;
 
 	/* Create the delegate constructor */
 	method = ILMethodCreate(classInfo, 0, ".ctor",
@@ -1720,6 +1893,10 @@ static void CreateDelegateMember(ILGenInfo *info, ILClass *classInfo,
 		 member->returnType,
 		 ILQualIdentSimple(ILInternString("Invoke", -1).string),
 		 member->params, 0);
+	
+	yysetfilename((ILNode*)decl,yygetfilename(member));
+	yysetlinenum((ILNode*)decl, yygetlinenum(member));
+
 	CreateMethod(info, classInfo, decl);
 	method = member->invokeMethod = decl->methodInfo;
 	if(method)
@@ -1729,6 +1906,167 @@ static void CreateDelegateMember(ILGenInfo *info, ILClass *classInfo,
 	}
 
 	/* TODO: asynchronous interface for delegates */
+	/* Clone the params list */
+	params=(ILNode_List*)ILNode_List_create();
+	AppendParameters(member->params,params,0);
+
+	ILNode_List_Add(params,
+						CreateFormalParameter("AsyncCallback","callback"));
+	ILNode_List_Add(params,
+						CreateFormalParameter("Object","object"));
+
+	/* Create the "BeginInvoke" method */
+	decl = (ILNode_MethodDeclaration*) ILNode_MethodDeclaration_create
+			(0,
+			IL_META_METHODDEF_PUBLIC |
+			IL_META_METHODDEF_VIRTUAL |
+			IL_META_METHODDEF_NEW_SLOT |
+			IL_META_METHODDEF_HIDE_BY_SIG |
+			IL_META_METHODDEF_COMPILER_CONTROLLED,
+			ILNode_SystemType_create("IAsyncResult"),
+			ILQualIdentSimple(ILInternString("BeginInvoke", -1).string),
+			(ILNode*)params,
+			0);
+
+	yysetfilename((ILNode*)decl,yygetfilename(member));
+	yysetlinenum((ILNode*)decl, yygetlinenum(member));
+
+	CreateMethod(info, classInfo, decl);
+	method = member->beginInvokeMethod = decl->methodInfo;
+	if(method)
+	{
+		ILMethodSetImplAttrs(method, ~((ILUInt32)0),
+							 IL_META_METHODIMPL_RUNTIME);
+	}
+
+	/* Clone the params list */
+	params=(ILNode_List*)ILNode_List_create();
+	
+	/* Append only the managed parameters */
+	AppendParameters(member->params,params,1);
+
+	ILNode_List_Add(params,
+						CreateFormalParameter("IAsyncResult","result"));
+
+	/* Create the "EndInvoke" method */
+	decl = (ILNode_MethodDeclaration*) ILNode_MethodDeclaration_create
+			(0,
+			IL_META_METHODDEF_PUBLIC |
+			IL_META_METHODDEF_VIRTUAL |
+			IL_META_METHODDEF_NEW_SLOT |
+			IL_META_METHODDEF_HIDE_BY_SIG |
+			IL_META_METHODDEF_COMPILER_CONTROLLED,
+			member->returnType,
+			ILQualIdentSimple(ILInternString("EndInvoke", -1).string),
+			(ILNode*)params,
+			0);
+	
+	yysetfilename((ILNode*)decl,yygetfilename(member));
+	yysetlinenum((ILNode*)decl, yygetlinenum(member));
+
+	CreateMethod(info, classInfo, decl);
+	method = member->endInvokeMethod = decl->methodInfo;
+	if(method)
+	{
+		ILMethodSetImplAttrs(method, ~((ILUInt32)0),
+							 IL_META_METHODIMPL_RUNTIME);
+	}
+}
+
+/*
+ * Check abstract method overrides.
+ */
+static void CheckAbstractOverrides(ILGenInfo *info, ILClass *classInfo,
+								   ILNode *node)
+{
+	ILClass *parent;
+	ILMethod *method;
+	ILClass *tempClass;
+	ILMethod *method2;
+
+	/* Scan up through the parents and look for all "abstract" methods */
+	parent = ILClass_Parent(classInfo);
+	while(parent != 0)
+	{
+		method = 0;
+		while((method = (ILMethod *)ILClassNextMemberByKind
+				(parent, (ILMember *)method, IL_META_MEMBERKIND_METHOD)) != 0)
+		{
+			/* Skip non-abstract methods */
+			if(!ILMethod_IsAbstract(method))
+			{
+				continue;
+			}
+
+			/* Scan from "classInfo" to look for an override for this method */
+			tempClass = classInfo;
+			method2 = 0;
+			while(tempClass != 0 && tempClass != parent)
+			{
+				method2 = (ILMethod *)ILClassNextMemberMatch
+					(tempClass, 0, IL_META_MEMBERKIND_METHOD,
+					 ILMethod_Name(method), ILMethod_Signature(method));
+				if(method2 && !ILMethod_IsNewSlot(method2))
+				{
+					break;
+				}
+				tempClass = ILClass_Parent(tempClass);
+				method2 = 0;
+			}
+
+			/* If we didn't find a match, then report an error */
+			if(!method2)
+			{
+				CCErrorOnLine(yygetfilename(node), yygetlinenum(node),
+							  "no override for abstract member `%s'",
+							  CSItemToName(ILToProgramItem(method)));
+			}
+		}
+		parent = ILClass_Parent(parent);
+	}
+}
+
+/*
+ * Fix up methods with the "public final virtual" attributes that
+ * we thought were interface member overrides that turned out not to
+ * be because there is an explicit override with the same signature.
+ */
+static void FixNonInterfaceMethods(ILClass *classInfo)
+{
+	ILMethod *method = 0;
+	ILOverride *override;
+	ILMethod *overMethod;
+	while((method = (ILMethod *)ILClassNextMemberByKind
+			(classInfo, (ILMember *)method, IL_META_MEMBERKIND_METHOD)) != 0)
+	{
+		/* Skip methods that aren't marked as "public final virtual" */
+		if(!ILMethod_IsPublic(method) || !ILMethod_IsFinal(method) ||
+		   !ILMethod_IsVirtual(method))
+		{
+			continue;
+		}
+
+		/* Look for an "Override" block with the same signature */
+		override = 0;
+		while((override = (ILOverride *)ILClassNextMemberByKind
+					(classInfo, (ILMember *)override,
+					 IL_META_MEMBERKIND_OVERRIDE)) != 0)
+		{
+			overMethod = ILOverrideGetDecl(override);
+			if(!strcmp(ILMethod_Name(overMethod), ILMethod_Name(method)) &&
+			   ILTypeIdentical(ILMethod_Signature(overMethod),
+			   				   ILMethod_Signature(method)))
+			{
+				/* We've found a match, so assume that the
+				   "final virtual" flags are incorrect */
+				ILMemberSetAttrs((ILMember *)method,
+								 IL_META_METHODDEF_VIRTUAL |
+								 IL_META_METHODDEF_NEW_SLOT |
+								 IL_META_METHODDEF_FINAL, 0);
+				break;
+			}
+		}
+	}
 }
 
 /*
@@ -1812,7 +2150,8 @@ static void CreateMembers(ILGenInfo *info, ILScope *globalScope,
 		}
 		else if(yykind(member) == yykindof(ILNode_ClassDefn))
 		{
-			CreateMembers(info, globalScope, member);
+			/* Create nested classes only after completing members of
+			 * the existing class to resolve overrides correctly */
 		}
 		else
 		{
@@ -1820,6 +2159,15 @@ static void CreateMembers(ILGenInfo *info, ILScope *globalScope,
 				  "internal error - do not know how to declare this member");
 		}
 	}
+	
+	ILNode_ListIter_Init(&iterator, body);
+	while((member = ILNode_ListIter_Next(&iterator)) != 0)
+	{
+		if(yykind(member) == yykindof(ILNode_ClassDefn))
+		{
+			CreateMembers(info, globalScope, member);
+		}
+	}
 
 	/* Add the "DefaultMember" attribute to the class if necessary */
 	if(defaultMemberName)
@@ -1827,6 +2175,17 @@ static void CreateMembers(ILGenInfo *info, ILScope *globalScope,
 		((ILNode_ClassDefn *)classNode)->defaultMemberName = defaultMemberName;
 	}
 
+	/* If the class is not abstract then make sure that all abstract
+	   methods in ancestor classes have been implemented here */
+	if(!ILClass_IsAbstract(classInfo))
+	{
+		CheckAbstractOverrides(info, classInfo, classNode);
+	}
+
+	/* Fix up "public final virtual" methods that we thought we
+	   interface implementations but which turn out not to be */
+	FixNonInterfaceMethods(classInfo);
+
 	/* Return to the original scope */
 	info->currentScope = savedScope;
 	info->currentClass = savedClass;
