@@ -80,6 +80,33 @@ extern	"C" {
 	{
 		return ILMemCmp(dst, src, len);
 	}
+#elif defined(CVM_ARM) && defined(__GNUC__) && !defined(IL_NO_ASM)
+    #define REGISTER_ASM_PC(x)              register x asm ("r4")
+    #define REGISTER_ASM_STACK(x)           register x asm ("r5")
+    #define REGISTER_ASM_FRAME(x)           register x asm ("r6")
+	#define	IL_MEMCPY(dst,src,len)			(ILMemCpy((dst), (src), (len)))
+	#define	IL_MEMMOVE(dst,src,len)			(ILMemMove((dst), (src), (len)))
+	#define	IL_MEMZERO(dst,len)				(ILMemZero((dst), (len)))
+	#define	IL_MEMSET(dst,ch,len)			(ILMemSet((dst), (ch), (len)))
+	#define	IL_MEMCMP(dst,src,len)			(ILMemCmp((dst), (src), (len)))
+#elif defined(CVM_PPC) && defined(__GNUC__) && !defined(IL_NO_ASM)
+    #define REGISTER_ASM_PC(x)              register x asm ("r18")
+    #define REGISTER_ASM_STACK(x)           register x asm ("r19")
+    #define REGISTER_ASM_FRAME(x)           register x asm ("r20")
+    #define IL_MEMCPY(dst,src,len)          (ILMemCpy((dst), (src), (len)))
+    #define IL_MEMMOVE(dst,src,len)         (ILMemMove((dst), (src), (len)))
+    #define IL_MEMZERO(dst,len)             (ILMemZero((dst), (len)))
+    #define IL_MEMSET(dst,ch,len)           (ILMemSet((dst), (ch), (len)))
+    #define IL_MEMCMP(dst,src,len)          (ILMemCmp((dst), (src), (len)))
+#elif defined(CVM_IA64) && defined(__GNUC__) && !defined(IL_NO_ASM)
+    #define REGISTER_ASM_PC(x)              register x asm ("r4")
+    #define REGISTER_ASM_STACK(x)           register x asm ("r5")
+    #define REGISTER_ASM_FRAME(x)           register x asm ("r6")
+	#define	IL_MEMCPY(dst,src,len)			(ILMemCpy((dst), (src), (len)))
+	#define	IL_MEMMOVE(dst,src,len)			(ILMemMove((dst), (src), (len)))
+	#define	IL_MEMZERO(dst,len)				(ILMemZero((dst), (len)))
+	#define	IL_MEMSET(dst,ch,len)			(ILMemSet((dst), (ch), (len)))
+	#define	IL_MEMCMP(dst,src,len)			(ILMemCmp((dst), (src), (len)))
 #else
     #define REGISTER_ASM_PC(x)              x
     #define REGISTER_ASM_STACK(x)           x
@@ -645,8 +672,8 @@ int _ILCVMInterpreter(ILExecThread *thread)
 					{
 						/* Unroll the current method to native code */
 					#ifdef IL_CVM_DIRECT_UNROLLED
-						if(_ILCVMUnrollMethod(thread->process->coder,
-											  pc, method))
+						if(_ILUnrollMethod(thread, thread->process->coder,
+										   pc, method))
 						{
 							VMSWITCH(0);
 						}
