@@ -319,6 +319,24 @@ static IL_INLINE int LURem(CVMWord *a, CVMWord *b)
 	}
 }
 
+static void LShiftLeft(CVMWord *a, ILUInt32 shiftBy)
+{
+	ILInt64 tempa = ReadLong(a);
+	WriteLong(a, tempa << shiftBy);
+}
+
+static void LShiftRight(CVMWord *a, ILUInt32 shiftBy)
+{
+	ILInt64 tempa = ReadLong(a);
+	WriteLong(a, tempa >> shiftBy);
+}
+
+static void LUShiftRight(CVMWord *a, ILUInt32 shiftBy)
+{
+	ILUInt64 tempa = ReadULong(a);
+	WriteULong(a, tempa >> shiftBy);
+}
+
 #elif defined(IL_CVM_LOCALS)
 
 /* No locals required */
@@ -1097,8 +1115,8 @@ VMBREAK(COP_LSUB);
 VMCASE(COP_LSUB_OVF):
 {
 	/* Long subtract with overflow detection */
-	if(!LSubOvf(&(stacktop[-(CVM_WORDS_PER_LONG * 2)]),
-		        &(stacktop[-CVM_WORDS_PER_LONG])))
+	if(LSubOvf(&(stacktop[-(CVM_WORDS_PER_LONG * 2)]),
+		       &(stacktop[-CVM_WORDS_PER_LONG])))
 	{
 		MODIFY_PC_AND_STACK(CVM_LEN_NONE, -CVM_WORDS_PER_LONG);
 	}
@@ -1141,8 +1159,8 @@ VMBREAK(COP_LSUB_OVF);
 VMCASE(COP_LSUB_OVF_UN):
 {
 	/* Unsigned long subtract with overflow detection */
-	if(!LUSubOvf(&(stacktop[-(CVM_WORDS_PER_LONG * 2)]),
-		         &(stacktop[-CVM_WORDS_PER_LONG])))
+	if(LUSubOvf(&(stacktop[-(CVM_WORDS_PER_LONG * 2)]),
+		        &(stacktop[-CVM_WORDS_PER_LONG])))
 	{
 		MODIFY_PC_AND_STACK(CVM_LEN_NONE, -CVM_WORDS_PER_LONG);
 	}
@@ -1222,8 +1240,8 @@ VMBREAK(COP_LMUL);
 VMCASE(COP_LMUL_OVF):
 {
 	/* Long multiply with overflow detection */
-	if(!LMulOvf(&(stacktop[-(CVM_WORDS_PER_LONG * 2)]),
-		        &(stacktop[-CVM_WORDS_PER_LONG])))
+	if(LMulOvf(&(stacktop[-(CVM_WORDS_PER_LONG * 2)]),
+		       &(stacktop[-CVM_WORDS_PER_LONG])))
 	{
 		MODIFY_PC_AND_STACK(CVM_LEN_NONE, -CVM_WORDS_PER_LONG);
 	}
@@ -1266,8 +1284,8 @@ VMBREAK(COP_LMUL_OVF);
 VMCASE(COP_LMUL_OVF_UN):
 {
 	/* Unsigned long multiply with overflow detection */
-	if(!LUMulOvf(&(stacktop[-(CVM_WORDS_PER_LONG * 2)]),
-		         &(stacktop[-CVM_WORDS_PER_LONG])))
+	if(LUMulOvf(&(stacktop[-(CVM_WORDS_PER_LONG * 2)]),
+		        &(stacktop[-CVM_WORDS_PER_LONG])))
 	{
 		MODIFY_PC_AND_STACK(CVM_LEN_NONE, -CVM_WORDS_PER_LONG);
 	}
@@ -2035,9 +2053,8 @@ VMBREAK(COP_LNOT);
 VMCASE(COP_LSHL):
 {
 	/* Long shift left */
-	WriteLong(&(stacktop[-(CVM_WORDS_PER_LONG + 1)]),
-		ReadLong(&(stacktop[-(CVM_WORDS_PER_LONG + 1)])) <<
-	    (stacktop[-1].uintValue & 0x3F));
+	LShiftLeft(&(stacktop[-(CVM_WORDS_PER_LONG + 1)]) ,
+				(stacktop[-1].uintValue & 0x3F));
 	MODIFY_PC_AND_STACK(CVM_LEN_NONE, -1);
 }
 VMBREAK(COP_LSHL);
@@ -2066,8 +2083,7 @@ VMBREAK(COP_LSHL);
 VMCASE(COP_LSHR):
 {
 	/* Long shift right */
-	WriteLong(&(stacktop[-(CVM_WORDS_PER_LONG + 1)]),
-		ReadLong(&(stacktop[-(CVM_WORDS_PER_LONG + 1)])) >>
+	LShiftRight(&(stacktop[-(CVM_WORDS_PER_LONG + 1)]) ,
 	    (stacktop[-1].uintValue & 0x3F));
 	MODIFY_PC_AND_STACK(CVM_LEN_NONE, -1);
 }
@@ -2096,8 +2112,7 @@ VMBREAK(COP_LSHR);
 VMCASE(COP_LSHR_UN):
 {
 	/* Unsigned long shift right */
-	WriteULong(&(stacktop[-(CVM_WORDS_PER_LONG + 1)]),
-		ReadULong(&(stacktop[-(CVM_WORDS_PER_LONG + 1)])) >>
+	LUShiftRight(&(stacktop[-(CVM_WORDS_PER_LONG + 1)]),
 	    (stacktop[-1].uintValue & 0x3F));
 	MODIFY_PC_AND_STACK(CVM_LEN_NONE, -1);
 }
