@@ -18,8 +18,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifndef IL_WITHOUT_TOOLS
+
 #include "il_dumpasm.h"
-#include "engine.h"
+#include "engine_private.h"
 
 #ifdef	__cplusplus
 extern	"C" {
@@ -55,6 +57,10 @@ extern	"C" {
 #define	CVM_OPER_LD_INTERFACE		24
 #define	CVM_OPER_TAIL_CALL			25
 #define	CVM_OPER_PACK_VARARGS		26
+#define	CVM_OPER_CUSTOM				27
+#define	CVM_OPER_TWO_UINT32			28
+#define	CVM_OPER_TAIL_INTERFACE		29
+#define	CVM_OPER_TYPE				30
 
 /*
  * Table of CVM opcodes.  This must be kept in sync with "cvm.h".
@@ -65,6 +71,7 @@ typedef struct
 	int			operands;
 
 } CVMOpcode;
+
 static CVMOpcode const opcodes[256] = {
 
 	/*
@@ -415,7 +422,7 @@ static CVMOpcode const opcodes[256] = {
 	 */
 	{"prefix",			CVM_OPER_PREFIX},
 };
-static CVMOpcode const prefixOpcodes[96] = {
+static CVMOpcode const prefixOpcodes[0x70] = {
 	/*
 	 * Reserved opcodes.
 	 */
@@ -454,6 +461,9 @@ static CVMOpcode const prefixOpcodes[96] = {
 	 * Prefixed call management opcodes.
 	 */
 	{"tail_call",		CVM_OPER_TAIL_CALL},
+	{"tail_calli",		CVM_OPER_NONE},
+	{"tail_callvirt",	CVM_OPER_TWO_UINT32},
+	{"tail_callintf",	CVM_OPER_TAIL_INTERFACE},
 	{"ldftn",			CVM_OPER_METHOD},
 	{"ldvirtftn",		CVM_OPER_UINT32},
 	{"ldinterfftn",		CVM_OPER_LD_INTERFACE},
@@ -465,11 +475,7 @@ static CVMOpcode const prefixOpcodes[96] = {
 	{"enter_try",		CVM_OPER_NONE},
 	{"throw",			CVM_OPER_NONE},
 	{"throw_caller",	CVM_OPER_NONE},
-
-	/*
-	 * Reserved opcodes.
-	 */
-	{"preserved_1e",	CVM_OPER_NONE},
+	{"set_stack_trace",	CVM_OPER_NONE},
 
 	/*
 	 * Prefixed typedref handling opcodes.
@@ -519,10 +525,17 @@ static CVMOpcode const prefixOpcodes[96] = {
 	{"str2utf8",		CVM_OPER_NONE},
 	{"ansi2str",		CVM_OPER_NONE},
 	{"utf82str",		CVM_OPER_NONE},
+	{"str2utf16",		CVM_OPER_NONE},
+	{"utf162str",		CVM_OPER_NONE},
 	{"delegate2fnptr",	CVM_OPER_NONE},
 	{"array2ptr",		CVM_OPER_NONE},
 	{"refarray2ansi",	CVM_OPER_NONE},
 	{"refarray2utf8",	CVM_OPER_NONE},
+	{"tocustom",		CVM_OPER_CUSTOM},
+	{"fromcustom",		CVM_OPER_CUSTOM},
+	{"array2ansi",		CVM_OPER_NONE},
+	{"array2utf8",		CVM_OPER_NONE},
+	{"struct2native",	CVM_OPER_TYPE},
 
 	/*
 	 * Inline method replacements.
@@ -536,6 +549,8 @@ static CVMOpcode const prefixOpcodes[96] = {
 	{"type_from_handle", CVM_OPER_NONE},
 	{"monitor_enter",	CVM_OPER_NONE},
 	{"monitor_exit",	CVM_OPER_NONE},
+	{"append_char",		CVM_OPER_METHOD},
+	{"is_white_space",	CVM_OPER_NONE},
 
 	/*
 	 * Binary value fixups.
@@ -553,24 +568,40 @@ static CVMOpcode const prefixOpcodes[96] = {
 	 */
 	{"local_alloc",		CVM_OPER_NONE},
 
+	/*
+	 * Method profiling.
+	 */
+	{"profile_count",	CVM_OPER_NONE},
+
+	/*
+	 * Thread static handling.
+	 */
+	{"thread_static",	CVM_OPER_TWO_UINT32},
+
+	/*
+	 * Argument packing for native calls.
+	 */
+	{"waddr_native_n",	CVM_OPER_TWO_UINT32},
+
 	/*
 	 * Reserved opcodes.
 	 */
-	{"preserved_51",	CVM_OPER_NONE},
-	{"preserved_52",	CVM_OPER_NONE},
-	{"preserved_53",	CVM_OPER_NONE},
-	{"preserved_54",	CVM_OPER_NONE},
-	{"preserved_55",	CVM_OPER_NONE},
-	{"preserved_56",	CVM_OPER_NONE},
-	{"preserved_57",	CVM_OPER_NONE},
-	{"preserved_58",	CVM_OPER_NONE},
-	{"preserved_59",	CVM_OPER_NONE},
-	{"preserved_5A",	CVM_OPER_NONE},
-	{"preserved_5B",	CVM_OPER_NONE},
-	{"preserved_5C",	CVM_OPER_NONE},
-	{"preserved_5D",	CVM_OPER_NONE},
-	{"preserved_5E",	CVM_OPER_NONE},
-	{"preserved_5F",	CVM_OPER_NONE},
+	{"preserved_60",	CVM_OPER_NONE},
+	{"preserved_61",	CVM_OPER_NONE},
+	{"preserved_62",	CVM_OPER_NONE},
+	{"preserved_63",	CVM_OPER_NONE},
+	{"preserved_64",	CVM_OPER_NONE},
+	{"preserved_65",	CVM_OPER_NONE},
+	{"preserved_66",	CVM_OPER_NONE},
+	{"preserved_67",	CVM_OPER_NONE},
+	{"preserved_68",	CVM_OPER_NONE},
+	{"preserved_69",	CVM_OPER_NONE},
+	{"preserved_6A",	CVM_OPER_NONE},
+	{"preserved_6B",	CVM_OPER_NONE},
+	{"preserved_6C",	CVM_OPER_NONE},
+	{"preserved_6D",	CVM_OPER_NONE},
+	{"preserved_6E",	CVM_OPER_NONE},
+	{"preserved_6F",	CVM_OPER_NONE},
 };
 
 /*
@@ -762,10 +793,17 @@ int _ILDumpCVMInsn(FILE *stream, ILMethod *currMethod, unsigned char *pc)
 
 		case CVM_OPER_CALL_INTERFACE:
 		{
+		#ifdef IL_USE_IMTS
+			method = (ILMethod *)CVMReadPointer(pc + 3);
+			ILDumpClassName(stream, ILProgramItem_Image(currMethod),
+							ILMethod_Owner(method), 0);
+			fprintf(stream, ", %d, %d", (int)(pc[1]), method->index);
+		#else
 			classInfo = (ILClass *)CVMReadPointer(pc + 3);
 			ILDumpClassName(stream, ILProgramItem_Image(currMethod),
 							classInfo, 0);
 			fprintf(stream, ", %d, %d", (int)(pc[1]), (int)(pc[2]));
+		#endif
 			size = 3 + sizeof(void *);
 		}
 		break;
@@ -856,12 +894,21 @@ int _ILDumpCVMInsn(FILE *stream, ILMethod *currMethod, unsigned char *pc)
 
 				case CVM_OPER_CALL_INTERFACE:
 				{
+				#ifdef IL_USE_IMTS
+					method = (ILMethod *)CVMReadPointer(pc + 10);
+					ILDumpClassName(stream, ILProgramItem_Image(currMethod),
+									ILMethod_Owner(method), 0);
+					fprintf(stream, ", %lu, %lu",
+							(unsigned long)IL_READ_UINT32(pc + 2),
+							(unsigned long)(method->index));
+				#else
 					classInfo = (ILClass *)CVMReadPointer(pc + 10);
 					ILDumpClassName(stream, ILProgramItem_Image(currMethod),
 									classInfo, 0);
 					fprintf(stream, ", %lu, %lu",
 							(unsigned long)IL_READ_UINT32(pc + 2),
 							(unsigned long)IL_READ_UINT32(pc + 6));
+				#endif
 					size = 10 + sizeof(void *);
 				}
 				break;
@@ -959,6 +1006,45 @@ int _ILDumpCVMInsn(FILE *stream, ILMethod *currMethod, unsigned char *pc)
 				}
 				break;
 
+				case CVM_OPER_CUSTOM:
+				{
+					putc('"', stream);
+					fwrite((void *)CVMReadPointer(pc + 6), 1,
+						   (unsigned)(IL_READ_UINT32(pc + 2)), stream);
+					putc('"', stream);
+				}
+				break;
+
+				case CVM_OPER_TWO_UINT32:
+				{
+					fprintf(stream, "%lu, %lu",
+							(unsigned long)(IL_READ_UINT32(pc + 2)),
+							(unsigned long)(IL_READ_UINT32(pc + 6)));
+					pc += 10;
+				}
+				break;
+
+				case CVM_OPER_TAIL_INTERFACE:
+				{
+				#ifdef IL_USE_IMTS
+					method = (ILMethod *)CVMReadPointer(pc + 10);
+					fprintf(stream, "%lu, %lu, ",
+							(unsigned long)(IL_READ_UINT32(pc + 2)),
+							(unsigned long)(method->index));
+					ILDumpClassName(stream, ILProgramItem_Image(currMethod),
+									ILMethod_Owner(method), 0);
+				#else
+					classInfo = (ILClass *)CVMReadPointer(pc + 10);
+					fprintf(stream, "%lu, %lu, ",
+							(unsigned long)(IL_READ_UINT32(pc + 2)),
+							(unsigned long)(IL_READ_UINT32(pc + 6)));
+					ILDumpClassName(stream, ILProgramItem_Image(currMethod),
+									classInfo, 0);
+				#endif
+					size = 10 + sizeof(void *);
+				}
+				break;
+
 				default:
 				{
 					size = 2;
@@ -1116,3 +1202,6 @@ int _ILDumpVarProfile(FILE *stream)
 #ifdef	__cplusplus
 };
 #endif
+
+#endif /* IL_WITHOUT_TOOLS */
+
