@@ -139,6 +139,15 @@ ILInt64 _IL_InfoMethods_GetWorkingSet(ILExecThread *thread)
 	return 0;
 }
 
+/*
+ * public static int GetProcessorCount();
+ */
+ILInt32 _IL_InfoMethods_GetProcessorCount(ILExecThread *thread)
+{
+	/* Simulate a uniprocessor machine */
+	return 1;
+}
+
 /*
  * public static String GetGlobalConfigDir();
  */
