@@ -1,7 +1,7 @@
 /*
  * link_library.c - Process libraries within a linker context.
  *
- * Copyright (C) 2001, 2003  Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2001, 2003, 2004  Southern Storm Software, Pty Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -129,14 +129,14 @@ char *ILLinkerResolveLibrary(ILLinker *linker, const char *name)
 		return newName;
 	}
 
-	/* Prepend "lib" and append the memory model.  This allows us
-	   to convert references such as "-lm" into "-llibm64" */
+	/* Prepend "lib".  This allows us to convert references
+	   such as "-lm" into "-llibm" */
 	if(strncmp(name, "lib", 3) == 0)
 	{
 		return 0;
 	}
 	len = strlen(name);
-	expanded = (char *)ILMalloc(len + 6);
+	expanded = (char *)ILMalloc(len + 4);
 	if(!expanded)
 	{
 		_ILLinkerOutOfMemory(linker);
@@ -144,14 +144,6 @@ char *ILLinkerResolveLibrary(ILLinker *linker, const char *name)
 	}
 	strcpy(expanded, "lib");
 	strcpy(expanded + 3, name);
-	if(linker->is32Bit)
-	{
-		strcpy(expanded + 3 + len, "32");
-	}
-	else
-	{
-		strcpy(expanded + 3 + len, "64");
-	}
 
 	/* Search the library directories for the expanded name */
 	newName = ILImageSearchPath
@@ -174,6 +166,7 @@ static void LibraryDestroy(ILLibrary *library)
 		next = library->altNames;
 		ILFree(library->name);
 		ILFree(library->filename);
+		ILFree(library->moduleName);
 		if(library->publicKey)
 		{
 			ILFree(library->publicKey);
@@ -325,6 +318,16 @@ static ILLibrary *ScanAssemblies(ILLinker *linker, ILContext *context,
 			LibraryDestroy(library);
 			return 0;
 		}
+		nextLibrary->moduleName = ILDupString(IL_LINKER_DLL_MODULE_NAME);
+		if(!(nextLibrary->moduleName))
+		{
+			ILFree(nextLibrary->filename);
+			ILFree(nextLibrary->name);
+			ILFree(nextLibrary);
+			_ILLinkerOutOfMemory(linker);
+			LibraryDestroy(library);
+			return 0;
+		}
 		ILMemCpy(nextLibrary->version, ILAssemblyGetVersion(assem),
 				 4 * sizeof(ILUInt16));
 		nextLibrary->altNames = 0;
@@ -332,13 +335,11 @@ static ILLibrary *ScanAssemblies(ILLinker *linker, ILContext *context,
 		{
 			nextLibrary->classHash = 0;
 			nextLibrary->symbolHash = 0;
-			nextLibrary->memoryModel = library->memoryModel;
-			nextLibrary->modelFlags = library->modelFlags;
+			nextLibrary->isCImage = library->isCImage;
 		}
 		else
 		{
-			nextLibrary->memoryModel =
-				ILLinkerCMemoryModel(image, &(nextLibrary->modelFlags));
+			nextLibrary->isCImage = ILLinkerIsCObject(image);
 			originator = ILAssemblyGetOriginator(assem, &originatorLen);
 			if(originator && originatorLen)
 			{
@@ -361,6 +362,7 @@ static ILLibrary *ScanAssemblies(ILLinker *linker, ILContext *context,
 				{
 					ILFree(nextLibrary->publicKey);
 				}
+				ILFree(nextLibrary->moduleName);
 				ILFree(nextLibrary->filename);
 				ILFree(nextLibrary->name);
 				ILFree(nextLibrary);
@@ -379,6 +381,7 @@ static ILLibrary *ScanAssemblies(ILLinker *linker, ILContext *context,
 				{
 					ILFree(nextLibrary->publicKey);
 				}
+				ILFree(nextLibrary->moduleName);
 				ILFree(nextLibrary->filename);
 				ILFree(nextLibrary->name);
 				ILFree(nextLibrary);
@@ -623,9 +626,30 @@ static int WalkTypeDefs(ILLinker *linker, ILImage *image, ILLibrary *library)
 			}
 
 			/* If this is the "<Module>" type, then walk the global symbols */
-			if((!strcmp(ILClass_Name(classInfo), IL_LINKER_EXE_MODULE_NAME) ||
-			    !strcmp(ILClass_Name(classInfo), IL_LINKER_DLL_MODULE_NAME)) &&
-			   ILClass_Namespace(classInfo) == 0)
+			if(_ILLinkerIsModule(classInfo))
+			{
+				if(library)
+				{
+					/* Set the module name for the library */
+					char *name = ILDupString(ILClass_Name(classInfo));
+					if(!name)
+					{
+						_ILLinkerOutOfMemory(linker);
+						return 0;
+					}
+					ILFree(library->moduleName);
+					library->moduleName = name;
+				}
+				if(!WalkGlobals(linker, image, library, classInfo))
+				{
+					return 0;
+				}
+				continue;
+			}
+
+			/* If this type is marked with "OpenSystem.C.GlobalScope", then
+			   it should also be inspected for global symbol definitions */
+			if(_ILLinkerIsGlobalScope(classInfo))
 			{
 				if(!WalkGlobals(linker, image, library, classInfo))
 				{
@@ -1080,11 +1104,18 @@ static ILMember *CreateSymbolRef(ILLinker *linker, ILLibrary *library,
 	}
 
 	/* Make a "TypeRef" for the library's "<Module>" type */
-	if(library)
+	if(libSymbol->member &&
+	   !_ILLinkerIsModule(ILMember_Owner(libSymbol->member)))
+	{
+		/* The symbol is in a C# class marked with "GlobalScope" */
+		classInfo = _ILLinkerConvertClassRef
+			(linker, ILMember_Owner(libSymbol->member));
+	}
+	else if(library)
 	{
 		find.linker = linker;
 		find.library = library;
-		libClass.name = IL_LINKER_DLL_MODULE_NAME;
+		libClass.name = library->moduleName;
 		libClass.namespace = 0;
 		libClass.parent = 0;
 		classInfo = MakeTypeRef(&find, &libClass, linker->image);
@@ -1210,6 +1241,25 @@ int _ILLinkerFindSymbol(ILLinker *linker, const char *name,
 	return 0;
 }
 
+void _ILLinkerUpdateSymbol(ILLinker *linker, const char *name,
+						   ILMember *member)
+{
+	ILLibrarySymbol *libSymbol;
+	libSymbol = ILHashFindType(linker->symbolHash, name, ILLibrarySymbol);
+	if(libSymbol && (libSymbol->flags & IL_LINKSYM_HAVE_REF) == 0)
+	{
+		libSymbol->member = member;
+		libSymbol->flags |= IL_LINKSYM_HAVE_REF;
+	}
+}
+
+int _ILLinkerHasSymbol(ILLinker *linker, const char *name)
+{
+	ILLibrarySymbol *libSymbol;
+	libSymbol = ILHashFindType(linker->symbolHash, name, ILLibrarySymbol);
+	return (libSymbol != 0);
+}
+
 int _ILLinkerCreateSymbolHash(ILLinker *linker)
 {
 	linker->symbolHash =
@@ -1227,23 +1277,30 @@ int _ILLinkerCreateSymbolHash(ILLinker *linker)
 
 void _ILLinkerAddSymbols(ILLinker *linker, ILImage *image)
 {
-	ILClass *classInfo;
-
-	/* Look for the "<Module>" type within the image */
-	classInfo = ILClassLookup(ILClassGlobalScope(image),
-							  IL_LINKER_EXE_MODULE_NAME, 0);
-	if(!classInfo)
+	ILClass *classInfo = 0;
+	while((classInfo = (ILClass *)ILImageNextToken
+				(image, IL_META_TOKEN_TYPE_DEF, (void *)classInfo)) != 0)
 	{
-		classInfo = ILClassLookup(ILClassGlobalScope(image),
-								  IL_LINKER_DLL_MODULE_NAME, 0);
-		if(!classInfo)
+		/* If this is the "<Module>" type, then walk the global symbols */
+		if(_ILLinkerIsModule(classInfo))
 		{
-			return;
+			if(!WalkGlobals(linker, image, 0, classInfo))
+			{
+				return;
+			}
+			continue;
+		}
+
+		/* If this type is marked with "OpenSystem.C.GlobalScope", then
+		   it should also be inspected for global symbol definitions */
+		if(_ILLinkerIsGlobalScope(classInfo))
+		{
+			if(!WalkGlobals(linker, image, 0, classInfo))
+			{
+				return;
+			}
 		}
 	}
-
-	/* Walk the symbol definitions in the "<Module>" type */
-	WalkGlobals(linker, image, 0, classInfo);
 }
 
 #ifdef	__cplusplus
