@@ -215,6 +215,7 @@ static void ReportResolveError(ILImage *image, ILToken token,
 		break;
 
 		case IL_META_TOKEN_TYPE_REF:
+		case IL_META_TOKEN_EXPORTED_TYPE:
 		{
 			/* The type was nested within another type */
 			ILClass *classInfo = ILClass_FromToken(image, token);
@@ -235,6 +236,21 @@ static void ReportResolveError(ILImage *image, ILToken token,
 			}
 		}
 		break;
+
+		case IL_META_TOKEN_FILE:
+		{
+			/* The type was imported from a file scope */
+			ILFileDecl *file = ILFileDecl_FromToken(image, token);
+			if(file)
+			{
+				importName = ILFileDecl_Name(file);
+				fprintf(stderr, "unresolved type: [.file %s]%s%s%s\n",
+						importName, (namespace ? namespace : ""),
+						(namespace ? "." : ""), name);
+				return;
+			}
+		}
+		break;
 	}
 
 	/* Leave off the scope */
@@ -383,6 +399,44 @@ ILUInt32 _ILSearchForRawToken(ILImage *image, ILSearchRawFunc func,
 	return 0;
 }
 
+/*
+ * Add a program item to the redo list.  Returns zero if out of memory.
+ */
+static int AddToRedoList(ILContext *context, ILProgramItem *item)
+{
+	ILProgramItem **items;
+	if(context->numRedoItems >= context->maxRedoItems)
+	{
+		items = (ILProgramItem **)ILRealloc
+			(context->redoItems,
+			 sizeof(ILProgramItem *) * (context->maxRedoItems + 64));
+		if(!items)
+		{
+			return 0;
+		}
+		context->redoItems = items;
+		context->maxRedoItems += 64;
+	}
+	context->redoItems[(context->numRedoItems)++] = item;
+	return 1;
+}
+
+/*
+ * Determine if a program item is currently on the redo list.
+ */
+static int IsOnRedoList(ILContext *context, ILProgramItem *item)
+{
+	ILUInt32 posn;
+	for(posn = 0; posn < context->numRedoItems; ++posn)
+	{
+		if(context->redoItems[posn] == item)
+		{
+			return 1;
+		}
+	}
+	return 0;
+}
+
 /*
  * Resolve the TypeRef's.  Returns zero if OK, -1 if a
  * second-phase scan is needed for TypeRef's to this module.
@@ -500,15 +554,38 @@ static int ResolveTypeRefs(ILImage *image, int loadFlags)
 					importInfo = ILClassLookup
 						(importScope, name, namespace);
 				}
+				else if(IsOnRedoList(image->context, scope))
+				{
+					/* The nesting parent is marked for redo,
+					   so mark the child for redo also */
+					if(!AddToRedoList(image->context, &(info->programItem)))
+					{
+						return IL_LOADERR_MEMORY;
+					}
+					continue;
+				}
 			}
 			break;
 
 			case IL_META_TOKEN_MODULE_REF:
 			{
-				/* Module reference imports are not currently
-				   supported.  Types should be imported from
-				   assemblies, not modules.  Module references
-				   should only be used for PInvoke imports */
+				/* Type is imported from a foreign module */
+				importImage = ILModuleToImage((ILModule *)scope);
+				if(importImage)
+				{
+					goto moduleImport;
+				}
+				if(!_ILImageDynamicLinkModule(image, image->filename,
+											  ILModule_Name((ILModule *)scope),
+											  loadFlags, &importImage) &&
+				   importImage != 0)
+				{
+					importImage = ILModuleToImage((ILModule *)scope);
+					if(importImage)
+					{
+						goto moduleImport;
+					}
+				}
 			}
 			break;
 
@@ -518,11 +595,31 @@ static int ResolveTypeRefs(ILImage *image, int loadFlags)
 				importImage = ILAssemblyToImage((ILAssembly *)scope);
 				if(importImage)
 				{
-					importScope = ILClassGlobalScope(importImage);
-					if(importScope)
+				moduleImport:
+					if(importImage == image || !(importImage->loading))
 					{
-						importInfo = ILClassLookup
-							(importScope, name, namespace);
+						importScope = ILClassGlobalScope(importImage);
+						if(importScope)
+						{
+							importInfo = ILClassLookup
+								(importScope, name, namespace);
+							if(!importInfo && image->context->numRedoItems != 0)
+							{
+								/* This reference may need to be redone */
+								goto redo;
+							}
+						}
+					}
+					else
+					{
+						/* We cannot resolve this TypeRef yet.  We queue it
+						   on the "redo" list to be redone later */
+					redo:
+						if(!AddToRedoList(image->context, &(info->programItem)))
+						{
+							return IL_LOADERR_MEMORY;
+						}
+						continue;
 					}
 				}
 			}
@@ -2455,8 +2552,24 @@ static int Load_MemberRef(ILImage *image, ILUInt32 *values,
 				if(!resolvedClass || ILClassIsRef(resolvedClass))
 				{
 					/* Failed to resolve the class */
-					member = 0;
-					break;
+					if(resolvedClass &&
+					   IsOnRedoList(image->context,
+					   			    &(resolvedClass->programItem)))
+					{
+						/* The TypeRef is on the redo list, so add
+						   the MemberRef to the redo list also */
+						if(!AddToRedoList(image->context,
+										  &(member->programItem)))
+						{
+							return IL_LOADERR_MEMORY;
+						}
+						break;
+					}
+					else
+					{
+						member = 0;
+						break;
+					}
 				}
 				resolvedMember = 0;
 				while(resolvedClass != 0)
@@ -3059,18 +3172,20 @@ static int Load_ExportedType(ILImage *image, ILUInt32 *values,
 				   	 		 void *userData)
 {
 	ILExportedType *type;
-	ILProgramItem *scope;
-	ILFileDecl *decl;
-	ILAssembly *assem;
-	ILExportedType *expType;
+	const char *namespace;
 
 	/* Create the exported type record */
+	namespace = ILImageGetString(image, values[IL_OFFSET_EXPTYPE_NAMESPACE]);
+	if(namespace && *namespace == '\0')
+	{
+		namespace = 0;
+	}
 	type = ILExportedTypeCreate(image, token,
 				  	values[IL_OFFSET_EXPTYPE_ATTRS],
 				    ILImageGetString
-						(image, values[IL_OFFSET_EXPTYPE_NAME]),
-				    ILImageGetString
-						(image, values[IL_OFFSET_EXPTYPE_NAMESPACE]));
+						(image, values[IL_OFFSET_EXPTYPE_NAME]), namespace,
+					ILProgramItem_FromToken
+						(image, values[IL_OFFSET_EXPTYPE_FILE]));
 	if(!type)
 	{
 		return IL_LOADERR_MEMORY;
@@ -3079,19 +3194,13 @@ static int Load_ExportedType(ILImage *image, ILUInt32 *values,
 	/* Set the foreign class identifier */
 	ILExportedTypeSetId(type, values[IL_OFFSET_EXPTYPE_CLASS]);
 
-	/* Set the scope */
-	scope = ILProgramItem_FromToken(image, values[IL_OFFSET_EXPTYPE_FILE]);
-	if((decl = ILProgramItemToFileDecl(scope)) != 0)
+	/* Add the exported type to the "redo" list */
+	if((image->loadFlags & IL_LOADFLAG_NO_RESOLVE) == 0)
 	{
-		ILExportedTypeSetScopeFile(type, decl);
-	}
-	else if((assem = ILProgramItemToAssembly(scope)) != 0)
-	{
-		ILExportedTypeSetScopeAssembly(type, assem);
-	}
-	else if((expType = ILProgramItemToExportedType(scope)) != 0)
-	{
-		ILExportedTypeSetScopeType(type, expType);
+		if(!AddToRedoList(image->context, &(type->classItem.programItem)))
+		{
+			return IL_LOADERR_MEMORY;
+		}
 	}
 
 	/* Done */
@@ -3343,6 +3452,10 @@ int _ILImageBuildMetaStructures(ILImage *image, const char *filename,
 	EXIT_IF_ERROR(LoadTokens(image, IL_META_TOKEN_TYPE_DEF,
 							 Load_TypeDefName, 0));
 
+	/* Load exported type declarations */
+	EXIT_IF_ERROR(LoadTokens(image, IL_META_TOKEN_EXPORTED_TYPE,
+							 Load_ExportedType, 0));
+
 	/* Load the assemblies that this image depends upon */
 	if((loadFlags & IL_LOADFLAG_NO_RESOLVE) == 0)
 	{
@@ -3405,10 +3518,6 @@ int _ILImageBuildMetaStructures(ILImage *image, const char *filename,
 	EXIT_IF_ERROR(LoadTokens(image, IL_META_TOKEN_GENERIC_PAR,
 							 Load_GenericPar, 0));
 
-	/* Load exported type declarations */
-	EXIT_IF_ERROR(LoadTokens(image, IL_META_TOKEN_EXPORTED_TYPE,
-							 Load_ExportedType, 0));
-
 	/* Done */
 	return 0;
 }
@@ -3536,6 +3645,251 @@ void *_ILImageLoadOnDemand(ILImage *image, ILToken token)
 	return 0;
 }
 
+/*
+ * Redo a "TypeRef" token that was left dangling because of
+ * recursive assembly references.
+ */
+static int RedoTypeRef(ILImage *image, ILClass *classInfo)
+{
+	ILProgramItem *scope = ILClassGetScope(classInfo);
+	ILProgramItem *importScope;
+	ILImage *importImage = 0;
+	ILClass *importInfo = 0;
+	switch(ILProgramItem_Token(scope) & IL_META_TOKEN_MASK)
+	{
+		case IL_META_TOKEN_TYPE_REF:
+		case IL_META_TOKEN_EXPORTED_TYPE:
+		{
+			/* Nested type within a foreign assembly */
+			importScope = _ILProgramItemLinkedTo(scope);
+			if(importScope)
+			{
+				importInfo = ILClassLookup
+					(importScope, ILClass_Name(classInfo),
+					 ILClass_Namespace(classInfo));
+			}
+		}
+		break;
+
+		case IL_META_TOKEN_MODULE_REF:
+		{
+			/* Type is imported from a foreign module */
+			importImage = ILModuleToImage((ILModule *)scope);
+		}
+		break;
+
+		case IL_META_TOKEN_ASSEMBLY_REF:
+		{
+			/* Type is imported from a foreign assembly */
+			importImage = ILAssemblyToImage((ILAssembly *)scope);
+		}
+		break;
+
+		case IL_META_TOKEN_FILE:
+		{
+			/* Type is imported from a foreign file */
+			importImage = ILFileDeclToImage((ILFileDecl *)scope);
+		}
+		break;
+	}
+	if(importImage)
+	{
+		importScope = ILClassGlobalScope(importImage);
+		if(importScope)
+		{
+			importInfo = ILClassLookup
+				(importScope, ILClass_Name(classInfo),
+				 ILClass_Namespace(classInfo));
+		}
+	}
+	if(importInfo)
+	{
+		/* Link "info" to "importInfo" */
+		if(!_ILProgramItemLink(&(classInfo->programItem),
+							   &(importInfo->programItem)))
+		{
+			return IL_LOADERR_MEMORY;
+		}
+		else
+		{
+			return 0;
+		}
+	}
+	else
+	{
+		/* Could not resolve the type */
+	#if IL_DEBUG_META
+		ReportResolveError(image, scope->token,
+						   ILClass_Name(classInfo),
+						   ILClass_Namespace(classInfo));
+	#endif
+		return IL_LOADERR_UNRESOLVED;
+	}
+}
+
+/*
+ * Redo a "MemberRef" token that was left dangling because of
+ * recursive assembly references.
+ */
+static int RedoMemberRef(ILImage *image, ILMember *member)
+{
+	ILToken token = ILMember_Token(member);
+	ILClass *classInfo = ILMember_Owner(member);
+	const char *name = ILMember_Name(member);
+	ILClass *resolvedClass;
+	ILMember *resolvedMember;
+	int isMethod = ILMember_IsMethod(member);
+	ILType *type = ILMember_Signature(member);
+	ILMethod *method;
+
+	/* Resolve the class the contains the member */
+	resolvedClass = ILClassResolve(classInfo);
+	if(!resolvedClass || ILClassIsRef(resolvedClass))
+	{
+		goto reportError;
+	}
+
+	/* Walk the class hierarchy looking for the member */
+	resolvedMember = 0;
+	while(resolvedClass != 0)
+	{
+		resolvedMember = 0;
+		while((resolvedMember = ILClassNextMember
+					(resolvedClass, resolvedMember)) != 0)
+		{
+			if(!strcmp(ILMember_Name(resolvedMember), name))
+			{
+				if(isMethod && ILMember_IsMethod(resolvedMember))
+				{
+					if(ILTypeIdentical
+						(ILMember_Signature(resolvedMember), type))
+					{
+						break;
+					}
+				}
+				else if(!isMethod && ILMember_IsField(resolvedMember))
+				{
+					if(ILTypeIdentical
+						(ILMember_Signature(resolvedMember), type))
+					{
+						break;
+					}
+				}
+			}
+		}
+		if(resolvedMember != 0)
+		{
+			break;
+		}
+		resolvedClass = ILClass_Parent(resolvedClass);
+	}
+	if(!resolvedMember)
+	{
+		/* Failed to resolve the member */
+		if(ILType_IsComplex(type) &&
+		   ILType_Kind(type) == (IL_TYPE_COMPLEX_METHOD |
+		   						 IL_TYPE_COMPLEX_METHOD_SENTINEL))
+		{
+			/* Create a local reference to a vararg call site */
+			method = ILMethodCreate(classInfo, token, name, 0);
+			if(!method)
+			{
+				return IL_LOADERR_MEMORY;
+			}
+			ILMethodSetCallConv(method, ILType_CallConv(type));
+			ILMemberSetSignature((ILMember *)method, type);
+			return 0;
+		}
+		goto reportError;
+	}
+	if(!_ILProgramItemLink(&(member->programItem),
+						   &(resolvedMember->programItem)))
+	{
+		return IL_LOADERR_MEMORY;
+	}
+	return 0;
+
+	/* Report an error in the member resolution process */
+reportError:
+#if IL_DEBUG_META
+	if(classInfo)
+	{
+		if(classInfo->className->namespace &&
+		   !strcmp(classInfo->className->namespace, "$Synthetic"))
+		{
+			fprintf(stderr,
+					"token 0x%08lX: member `%s.%s' not found\n",
+					(unsigned long)token, classInfo->className->name, name);
+		}
+		else
+		{
+			fprintf(stderr,
+					"token 0x%08lX: member `%s%s%s.%s' not found\n",
+					(unsigned long)token,
+					(classInfo->className->namespace
+							? classInfo->className->namespace : ""),
+					(classInfo->className->namespace ? "." : ""),
+					classInfo->className->name, name);
+		}
+	}
+	else
+	{
+		fprintf(stderr,
+				"token 0x%08lX: member %s not found\n",
+				(unsigned long)token, name);
+	}
+#endif
+	return IL_LOADERR_UNRESOLVED;
+}
+
+int _ILImageRedoReferences(ILContext *context)
+{
+	ILUInt32 posn;
+	ILProgramItem *item;
+	int error = 0;
+	int error2;
+
+	/* Process the TypeRef's that are registered to be redone */
+	for(posn = 0; posn < context->numRedoItems; ++posn)
+	{
+		item = context->redoItems[posn];
+		switch(item->token & IL_META_TOKEN_MASK)
+		{
+			case IL_META_TOKEN_TYPE_REF:
+			case IL_META_TOKEN_EXPORTED_TYPE:
+			{
+				error2 = RedoTypeRef(item->image, (ILClass *)item);
+				if(!error)
+				{
+					error = error2;
+				}
+			}
+			break;
+		}
+	}
+
+	/* Process the MemberRef's that are registered to be redone.
+	   Must be done after all TypeRef's because a member's signature
+	   may refer to a TypeRef that appears later in the redo list */
+	for(posn = 0; posn < context->numRedoItems; ++posn)
+	{
+		item = context->redoItems[posn];
+		switch(item->token & IL_META_TOKEN_MASK)
+		{
+			case IL_META_TOKEN_MEMBER_REF:
+			{
+				error2 = RedoMemberRef(item->image, (ILMember *)item);
+				if(!error)
+				{
+					error = error2;
+				}
+			}
+			break;
+		}
+	}
+	return error;
+}
+
 #ifdef	__cplusplus
 };
 #endif
