@@ -22,11 +22,17 @@
 #define	_IL_ENGINE_H
 
 #include "il_program.h"
+#include "il_thread.h"
 
 #ifdef	__cplusplus
 extern	"C" {
 #endif
 
+/*
+ *	Structure used by the EE for storing monitors.
+ */
+typedef struct _tagILExecMonitor ILExecMonitor;
+
 /*
  * Execution control context for a process.
  */
@@ -176,10 +182,25 @@ typedef int (*ILExecDebugHookFunc)(void *userData,
  */
 void ILExecInit(unsigned long maxSize);
 
+/*
+ *	Deinitialize the engine.
+ */
+void ILExecDeinit();
+
+/*
+ *	Registers an ILThread and allow it to execute managed code. 
+ */
+ILExecThread *ILThreadRegisterForManagedExecution(ILExecProcess *process, ILThread *thread);
+
+/*
+ *	Unregisters an ILThread that no longer needs to execute managed code.
+ */
+void ILThreadUnregisterForManagedExecution(ILThread *thread);
+
 /*
  * Create a new process, including the "main" thread.
  */
-ILExecProcess *ILExecProcessCreate(unsigned long frameStackSize);
+ILExecProcess *ILExecProcessCreate(unsigned long frameStackSize, unsigned long cachePageSize);
 
 /*
  * Destroy a process and all threads associated with it.
@@ -196,6 +217,12 @@ void ILExecProcessSetLibraryDirs(ILExecProcess *process,
 								 char **libraryDirs,
 								 int numLibraryDirs);
 
+/*
+ *Set the flags for profiling, debugging etc
+ */
+void ILExecProcessSetCoderFlags(ILExecProcess *process,
+								int flags);
+
 /*
  * Get the IL context associated with a process.
  */
@@ -206,12 +233,16 @@ ILContext *ILExecProcessGetContext(ILExecProcess *process);
  */
 ILExecThread *ILExecProcessGetMain(ILExecProcess *process);
 
+#ifndef REDUCED_STDIO
+
 /*
  * Load an image into a process.  Returns 0 if OK, or
  * an image load error code otherwise (see "il_image.h").
  */
 int ILExecProcessLoadImage(ILExecProcess *process, FILE *file);
 
+#endif
+
 /*
  * Load the contents of a file as an image into a process.
  * Returns 0 if OK, -1 if the file could not be opened,
@@ -302,17 +333,6 @@ int ILExecProcessAddInternalCallTable(ILExecProcess* process,
  */
 ILExecThread *ILExecThreadCurrent(void);
 
-/*
- * Create a new thread and attach it to a process.
- * Returns NULL if out of memory.
- */
-ILExecThread *ILExecThreadCreate(ILExecProcess *process);
-
-/*
- * Destroy a thread and detach it from its process.
- */
-void ILExecThreadDestroy(ILExecThread *thread);
-
 /*
  * Get the process that corresponds to a thread.
  */
@@ -353,6 +373,15 @@ ILMethod *ILExecThreadLookupMethod(ILExecThread *thread,
 								   const char *methodName,
 								   const char *signature);
 
+/*
+ * Look up a method in a particular class.  Returns NULL if
+ * the method could not be found.
+ */
+ILMethod *ILExecThreadLookupMethodInClass(ILExecThread *thread,
+										  ILClass *classInfo,
+								   		  const char *methodName,
+								   		  const char *signature);
+
 /*
  * Look up a field by type name, field name, and signature.
  * Returns NULL if the field could not be found.  This function
@@ -594,6 +623,13 @@ ILString *ILStringIntern(ILExecThread *thread, ILString *str);
  */
 char *ILStringToUTF8(ILExecThread *thread, ILString *str);
 
+/*
+ * Convert a string into a NUL-terminated UTF-16 buffer,
+ * allocated within the garbage collected heap.  If "str"
+ * is NULL, then NULL will be returned.
+ */
+ILUInt16 *ILStringToUTF16(ILExecThread *thread, ILString *str);
+
 /*
  * Convert a string into an "ANSI" string in the local
  * character set, allocated within the garbage collected heap.
@@ -601,6 +637,13 @@ char *ILStringToUTF8(ILExecThread *thread, ILString *str);
  */
 char *ILStringToAnsi(ILExecThread *thread, ILString *str);
 
+/*
+ * Convert a string into "ANSI" and normalize any pathname
+ * separators to the system's standard character.  If "str"
+ * is NULL, then NULL will be returned.
+ */
+char *ILStringToPathname(ILExecThread *thread, ILString *str);
+
 /*
  * Get an element of an array.  Returns non-zero if an exception occurred.
  */
@@ -618,6 +661,12 @@ int ILExecThreadSetElem(ILExecThread *thread, ILObject *array,
  */
 ILObject *ILExecThreadBox(ILExecThread *thread, ILType *type, void *ptr);
 
+/*
+ * Box the value at a pointer using a specific primitive or value type,
+ * and handle the case where float/double values are stored as ILNativeFloat.
+ */
+ILObject *ILExecThreadBoxFloat(ILExecThread *thread, ILType *type, void *ptr);
+
 /*
  * Unbox an object using a specific primitive or value type and
  * write the contents to a pointer.  Returns zero if the object
@@ -626,6 +675,22 @@ ILObject *ILExecThreadBox(ILExecThread *thread, ILType *type, void *ptr);
 int ILExecThreadUnbox(ILExecThread *thread, ILType *type,
 					  ILObject *object, void *ptr);
 
+/*
+ * Unbox an object using a specific primitive or value type and
+ * write the contents to a pointer and handle the case where
+ * float/double values are stored as ILNativeFloat.
+ */
+int ILExecThreadUnboxFloat(ILExecThread *thread, ILType *type,
+					       ILObject *object, void *ptr);
+
+/*
+ * Box the value at a pointer using specific primitive or subsequent 
+ * promotions. Handle the cases where an Int16 is being unboxed to an 
+ * Int32 or Int64.
+ */
+int ILExecThreadPromoteAndUnbox(ILExecThread *thread, ILType *type, 
+								ILObject *object, void *ptr);
+
 /*
  * Print the current exception object to standard error.
  */
