@@ -20,20 +20,35 @@
 
 #include "il_system.h"
 #include "il_utils.h"
+#ifdef IL_WIN32_NATIVE
+#include <winsock.h>
+#define	HAVE_GETHOSTBYNAME	1
+#define	HAVE_GETHOSTBYADDR	1
+#else
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
 #if HAVE_NETDB_H
 #include <netdb.h>
 #endif 
+#endif 
 
 #ifdef	__cplusplus
 extern	"C" {
 #endif
 
+#ifdef IL_WIN32_NATIVE
+
+void _ILWinSockInit(void);
+
+#endif
+
 struct hostent* ILGetHostByName(const char *name)
 {
 #ifdef HAVE_GETHOSTBYNAME
+#ifdef IL_WIN32_NATIVE
+	_ILWinSockInit();
+#endif
 	return gethostbyname(name);
 #else
 	return NULL;
@@ -44,6 +59,9 @@ struct hostent* ILGetHostByName(const char *name)
 struct hostent* ILGetHostByAddr(const void* addr, unsigned int len,int type)
 {
 #ifdef HAVE_GETHOSTBYADDR
+#ifdef IL_WIN32_NATIVE
+	_ILWinSockInit();
+#endif
 	return gethostbyaddr(addr,len,type);
 #else
 	return NULL;
