@@ -20,9 +20,13 @@
 
 #include "ilunit.h"
 #include "il_thread.h"
+#include "il_gc.h"
 #if HAVE_UNISTD_H
 	#include <unistd.h>
 #endif
+#ifdef IL_WIN32_NATIVE
+	#include <windows.h>
+#endif
 
 #ifdef	__cplusplus
 extern	"C" {
@@ -37,11 +41,17 @@ static void sleepFor(int steps)
 	/* Time steps are 100ms in length */
 	usleep(steps * 100000);
 #define	STEPS_TO_MS(steps)	(steps * 100)
+#else
+#ifdef IL_WIN32_NATIVE
+	/* Time steps are 100ms in length */
+	Sleep(steps * 100);
+#define	STEPS_TO_MS(steps)	(steps * 100)
 #else
 	/* Time steps are 1s in length */
 	sleep(steps);
 #define	STEPS_TO_MS(steps)	(steps * 1000)
 #endif
+#endif
 }
 
 /*
@@ -1080,9 +1090,10 @@ static void thread_counts(void *arg)
 	{
 		ILUnitFailed("foreground thread count has not returned to 1");
 	}
-	if(numBackground != 0)
+	if(numBackground != 1)
 	{
-		ILUnitFailed("background thread count has not returned to 0");
+		/* Currently there is one background thread (the finalizer thread) */
+		ILUnitFailed("background thread count has not returned to 1");
 	}
 }
 
@@ -1166,6 +1177,134 @@ static void mutex_create(void *arg)
 	ILWaitHandleClose(handle2);
 }
 
+/*
+ * Test monitor creation.
+ */
+static void monitor_create(void *arg)
+{
+	ILWaitHandle *handle;
+	handle = ILWaitMonitorCreate();
+	if(!handle)
+	{
+		ILUnitFailed("could not create a monitor");
+	}
+	ILWaitHandleClose(handle);
+}
+
+/*
+ * Test monitor acquire/release.
+ */
+static void monitor_acquire(void *arg)
+{
+	ILWaitHandle *handle;
+
+	/* Create the monitor */
+	handle = ILWaitMonitorCreate();
+	if(!handle)
+	{
+		ILUnitFailed("could not create a monitor");
+	}
+
+	/* Acquire it: zero timeout but we should get it immediately */
+	if(ILWaitMonitorTryEnter(handle, 0) != 0)
+	{
+		ILUnitFailed("could not acquire (1)");
+	}
+
+	/* Acquire it again */
+	if(ILWaitMonitorEnter(handle) != 0)
+	{
+		ILUnitFailed("could not acquire (2)");
+	}
+
+	/* Release twice */
+	if(!ILWaitMonitorLeave(handle))
+	{
+		ILUnitFailed("could not release (1)");
+	}
+	if(!ILWaitMonitorLeave(handle))
+	{
+		ILUnitFailed("could not release (2)");
+	}
+
+	/* Try to release again, which should fail */
+	if(ILWaitMonitorLeave(handle) != 0)
+	{
+		ILUnitFailed("released a monitor that we don't own");
+	}
+
+	/* Clean up */
+	ILWaitHandleClose(handle);
+}
+
+/*
+ * Thread start function that holds a monitor for a period of time.
+ */
+static void monitorHold(void *arg)
+{
+	ILWaitHandle *monitor = ILWaitMonitorCreate();
+	ILWaitMonitorEnter(monitor);
+	sleepFor(2);
+	globalFlag = 1;
+	ILWaitMonitorLeave(monitor);
+	ILWaitHandleClose(monitor);
+	sleepFor(2);
+}
+
+/*
+ * Test that a thread can be suspended while it holds a monitor.
+ */
+static void monitor_suspend(void *arg)
+{
+	ILThread *thread;
+	int savedFlag;
+
+	/* Create the thread */
+	thread = ILThreadCreate(monitorHold, 0);
+	if(!thread)
+	{
+		ILUnitOutOfMemory();
+	}
+
+	/* Clear the global flag */
+	globalFlag = 0;
+
+	/* Start the thread, which should immediately suspend */
+	ILThreadStart(thread);
+
+	/* Wait 1 time step */
+	sleepFor(1);
+
+	/* Suspend the thread */
+	ILThreadSuspend(thread);
+
+	/* Wait for 4 time steps (enough for the thread to exit
+	   if it wasn't suspended) */
+	sleepFor(4);
+
+	/* Save the global flag at this point */
+	savedFlag = globalFlag;
+
+	/* Resume the thread */
+	ILThreadResume(thread);
+
+	/* Wait 4 more time steps for the thread to exit */
+	sleepFor(4);
+
+	/* Clean up the thread object (the thread itself is now dead) */
+	ILThreadDestroy(thread);
+
+	/* Check for errors: the flag must not have been set */
+	if(savedFlag)
+	{
+		ILUnitFailed("thread holding the monitor did not suspend");
+	}
+	if(!globalFlag)
+	{
+		ILUnitFailed("thread holding the monitor did not finish");
+	}
+}
+
 /*
  * Simple test registration macro.
  */
@@ -1190,6 +1329,11 @@ void ILUnitRegisterTests(void)
 	 */
 	ILThreadInit();
 
+	/*
+	 * Initialize the GC system (the GC is used to create threads).
+	 */
+	ILGCInit(0);	
+
 	/*
 	 * Test the properties of the "main" thread.
 	 */
@@ -1243,6 +1387,14 @@ void ILUnitRegisterTests(void)
 	 */
 	ILUnitRegisterSuite("Wait Mutex Tests");
 	RegisterSimple(mutex_create);
+
+	/*
+	 * Test monitor behaviours.
+	 */
+	ILUnitRegisterSuite("Monitor Tests");
+	RegisterSimple(monitor_create);
+	RegisterSimple(monitor_acquire);
+	RegisterSimple(monitor_suspend);
 }
 
 #ifdef	__cplusplus
